﻿#Region "LICENSE"
'
' Copyright (c) Microncode.com
' 
' This source is subject to the Microsoft Public License. 
' See https://opensource.org/licenses/ms-pl.html.
' All other rights reserved. 
' 
' THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
' WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
'
#End Region

Imports System
Imports System.Drawing
Imports System.Reflection
Imports System.Windows.Forms
Imports CSScreenRecorder

''' <summary>
''' The main example of using the CSScreenRecorder library.
''' </summary>
Public Class frmMain
    Inherits Form

    'Create a new instance of the library
    Private ScreenRecorder As ScreenRecorder

    Private Sub frmMain_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        'Add rotate style
        Me.cboRotate.Items.Add("RotateNoneFlipNone")
        Me.cboRotate.Items.Add("RotateNoneFlipXY")
        Me.cboRotate.Items.Add("RotateNoneFlipY")

        'Set the rotate to none
        Me.cboRotate.SelectedIndex = 0


        'Set the spinning tool style
        For Each prop As PropertyInfo In GetType(Color).GetProperties()
            If Equals(prop.PropertyType.FullName, "System.Drawing.Color") Then Me.cboSpinningToolColor.Items.Add(prop.Name)
        Next

        Me.cboSpinningToolColor.Text = "Red"
        Me.cboSpinningToolStyle.Text = "Dot"

        Using ScreenRecorder = New ScreenRecorder("UserName", "RegKey")

            'Get the available codecs
            For Each Codec As ScreenRecorder.Codecs In ScreenRecorder.GetCodecs()
                Console.WriteLine(Codec.CodecName)
                Me.cboVideoCodecs.Items.Add(Codec.CodecName)
            Next

            Me.cboVideoCodecs.SelectedIndex = 1

#Region "LoadAudioProperties"

            'Formats:
            Me.cboAudioFormat.DataSource = ScreenRecorder.GetAudioFormats()
            Me.cboAudioFormat.Text = "MP3"

            'Bitrate
            Me.cboBitrate.DataSource = ScreenRecorder.GetAudioBitrates()
            Me.cboBitrate.Text = "192"

            'Samplerate
            Me.cboSamplerate.DataSource = ScreenRecorder.GetAudioSamplerates()
            Me.cboSamplerate.Text = "44100"

            'Bit depth
            Me.cboBits.DataSource = ScreenRecorder.GetAudioBits()
            Me.cboBits.Text = "16"

            'Channels
            Me.cboChannels.DataSource = ScreenRecorder.GetAudioChannels()
            Me.cboChannels.Text = "2"

            'Recorder mode
            Me.cboRecorderMode.DataSource = ScreenRecorder.GetAudioModes()
            Me.cboRecorderMode.SelectedItem = ScreenRecorder.GetEnumValue(ScreenRecorder.AudioMode)

#End Region

        End Using

        'Set the area to record to a full screen
        Me.txtTop.Text = "0"
        Me.txtLeft.Text = "0"

        'Full screen width and height
        Me.txtWidth.Text = Screen.PrimaryScreen.Bounds.Width.ToString()
        Me.txtHeight.Text = Screen.PrimaryScreen.Bounds.Height.ToString()
    End Sub

    Private Sub btnSpinningTool_Click(sender As Object, e As EventArgs) Handles btnSpinningTool.Click
        Dim SpinningSelection As SpinningSelection = New SpinningSelection()


        'In order to get your own UserName and RegKey please order a license.
        Using ScreenRecorder = New ScreenRecorder("UserName", "RegKey")
            'Set the spinning tool border color
            ScreenRecorder.SpinningToolColor = Color.FromName(Me.cboSpinningToolColor.SelectedItem.ToString())

            'Set the spinning tool border size
            ScreenRecorder.SpinningToolBorderSize = Integer.Parse(Me.txtSpinningToolBorder.Text)

            'Set the spinning tool border style
            ScreenRecorder.SpinningToolStyle = CType(Me.cboSpinningToolStyle.SelectedIndex, Drawing2D.DashStyle)

            'On Spinning Selection
            AddHandler ScreenRecorder.OnSpinningSelection, Sub(o, Cancel, Top, Left, Width, Height)
                                                               'Spinning tool selection event:
                                                               If Cancel Then
                                                                   Me.txtLog.Text = Me.txtLog.Text & Date.Now & ":Spinner tool canceled" & Microsoft.VisualBasic.Constants.vbCrLf
                                                               Else
                                                                   Me.txtTop.Text = Top.ToString()
                                                                   Me.txtLeft.Text = Left.ToString()
                                                                   Me.txtWidth.Text = Width.ToString()
                                                                   Me.txtHeight.Text = Height.ToString()
                                                                   Me.txtLog.Text = Me.txtLog.Text & Date.Now & ":Spinner tool selected - " & Top & " " & Left & " " & Width & " " & Height & Microsoft.VisualBasic.Constants.vbCrLf
                                                               End If
                                                           End Sub


            'Display the spinning tool
            SpinningSelection = ScreenRecorder.SpinningTool()
        End Using
    End Sub

    Private Sub timer1_Tick(sender As Object, e As EventArgs) Handles timer1.Tick
        'Get active window
        Dim activewindow As ActiveWindow = New ActiveWindow()
        activewindow = ScreenRecorder.GetActiveWindow()

        'Set the active window coordinates
        Me.txtTop.Text = activewindow.Top.ToString()
        Me.txtLeft.Text = activewindow.Left.ToString()
        Me.txtWidth.Text = activewindow.Width.ToString()
        Me.txtHeight.Text = activewindow.Height.ToString()
    End Sub

    Private Sub btnActiveWindow_Click(sender As Object, e As EventArgs) Handles btnActiveWindow.Click
        If Not Me.timer1.Enabled Then
            'Start get active window
            Me.timer1.Enabled = True
            Me.btnActiveWindow.Text = "Stop"
        Else
            'Stop get active window
            Me.timer1.Enabled = False
            Me.btnActiveWindow.Text = "Get active window"
        End If
    End Sub

    <Obsolete>
    Private Sub cmdRecord_Click(sender As Object, e As EventArgs) Handles cmdRecord.Click

        'Using ScreenRecorder = New ScreenRecorder("UserName", "RegKey")
        Using CSharpImpl.__Assign(ScreenRecorder, New ScreenRecorder("UserName", "RegKey"))
#Region "Area"

            'Set the area to record
            ScreenRecorder.RecordScreenHeight = Integer.Parse(Me.txtHeight.Text)
            ScreenRecorder.RecordScreenWidth = Integer.Parse(Me.txtWidth.Text)
            ScreenRecorder.RecordScreenTop = Integer.Parse(Me.txtTop.Text)
            ScreenRecorder.RecordScreenLeft = Integer.Parse(Me.txtLeft.Text)

#End Region

#Region "Video"

            'Set the frames rate
            ScreenRecorder.VideoFPS = 10
            ScreenRecorder.VideoEncoderQuality = Me.trcVideoEncoderQuality.Value
            Me.picPreview.SizeMode = PictureBoxSizeMode.StretchImage

            'Set the video codec
            ScreenRecorder.VideoCodecIndex = Me.cboVideoCodecs.SelectedIndex

#End Region

#Region "Audio"

            'Record audio?
            ScreenRecorder.AudioRecorder = Me.chkRecordAudio.Checked

            'Set the audio device index to record from:
            ScreenRecorder.AudioDeviceIndex = Me.cboAudioSource.SelectedIndex

            '[Optional: Default is WasapiLoopbackCapture]
            'The mode of the recording process. This can be WasapiLoopbackCapture(default), WasapiCapture or LineIn:
            ScreenRecorder.AudioMode = CType([Enum].Parse(GetType(CSScreenAudioRecorder.Mode), Me.cboRecorderMode.Text), CSScreenAudioRecorder.Mode)

            '[Optional: Default is 44100]
            'Set the audio sample rate to record from the device and the destination file:
            ScreenRecorder.AudioSamplerate = CType([Enum].Parse(GetType(CSScreenAudioRecorder.Samplerate), Me.cboSamplerate.Text), CSScreenAudioRecorder.Samplerate)
            'Or set by value
            'audioRecorder1.SamplerateVal = 48000;

            '[Optional: Default is 16]
            'Set the bit-depth to record from the device and the destination file:
            ScreenRecorder.AudioBits = CType([Enum].Parse(GetType(CSScreenAudioRecorder.Bits), Me.cboBits.Text), CSScreenAudioRecorder.Bits)

            '[Optional: Default is 2]
            'Set the number of the channels to record from the device and the destination file:
            ScreenRecorder.AudioChannels = CType([Enum].Parse(GetType(CSScreenAudioRecorder.Channels), Me.cboChannels.Text), CSScreenAudioRecorder.Channels)

            '[Optional: Default is MP3]
            'The audio format of the destination file, this can be MP3 or WAV(PCM).
            ScreenRecorder.AudioFormat = CType([Enum].Parse(GetType(CSScreenAudioRecorder.Format), Me.cboAudioFormat.Text), CSScreenAudioRecorder.Format)

            '[Optional: Default is 192]
            'Set the bitrate of the MP3 encoder:
            ScreenRecorder.AudioBitrate = CType([Enum].Parse(GetType(CSScreenAudioRecorder.Bitrate), Me.cboBitrate.Text), CSScreenAudioRecorder.Bitrate)
            'Or set by value
            'audioRecorder1.BitrateVal = 192;
#End Region

#Region "Style"

            'Track mouse move
            ScreenRecorder.TrackMouse = Me.chkTrackMouse.Checked

            'Set the rotate style
            ScreenRecorder.Rotate = RotateStyle.RotateNoneFlipNone
            If Equals(Me.cboRotate.Text, "RotateNoneFlipXY") Then ScreenRecorder.Rotate = RotateStyle.RotateNoneFlipXY
            If Equals(Me.cboRotate.Text, "RotateNoneFlipY") Then ScreenRecorder.Rotate = RotateStyle.RotateNoneFlipY

            'Set the Watermark
            ScreenRecorder.WatermarkFont = New Font(New Font("Arial", 32), FontStyle.Underline)
            ScreenRecorder.WatermarkText = Me.txtWatermarkText.Text

#End Region

#Region "Events"

            'On start event
            AddHandler ScreenRecorder.OnStartRecord, Sub(o)
                                                         Me.lblPreview.Visible = False
                                                         Me.txtLog.Text = Me.txtLog.Text & Date.Now & ":Start record" & Microsoft.VisualBasic.Constants.vbCrLf
                                                     End Sub


            'On pause record
            AddHandler ScreenRecorder.OnPauseRecord, Sub(o) Me.txtLog.Text = Me.txtLog.Text & Date.Now & ":Pause record" & Microsoft.VisualBasic.Constants.vbCrLf

            'On unpause record
            AddHandler ScreenRecorder.OnUnPauseRecord, Sub(o) Me.txtLog.Text = Me.txtLog.Text & Date.Now & ":Unpause record" & Microsoft.VisualBasic.Constants.vbCrLf

            'On progress
            AddHandler ScreenRecorder.OnProgress, Sub(o, i)
                                                      Console.WriteLine(i.ToString())
                                                  End Sub

            'On done  record
            AddHandler ScreenRecorder.OnDoneRecord, Sub(o)
                                                        Me.lblPreview.Visible = True
                                                        Me.txtLog.Text = Me.txtLog.Text & Date.Now & ":Stop record" & Microsoft.VisualBasic.Constants.vbCrLf
                                                    End Sub
            'On stop  record
            AddHandler ScreenRecorder.OnStopRecord, Sub(o)
                                                    End Sub

            'On preview (get the bmp image)
            AddHandler ScreenRecorder.OnPreview, Sub(o, bmp)
                                                     'Preview
                                                     Dim pImage As Bitmap = New Bitmap(bmp)
                                                     Me.picPreview.Image = pImage
                                                 End Sub

            AddHandler ScreenRecorder.OnError, Sub(o, ErrorMessage, ErrorNumber) Me.txtLog.Text = Me.txtLog.Text & Date.Now & ":Error - " & ErrorMessage & " (" & ErrorNumber & ")" & Microsoft.VisualBasic.Constants.vbCrLf

#End Region

            'Set the file name
            Me.txtFileName.Text = Application.StartupPath & "\" & Date.Now.ToString("yyyy-MM-dd-HH-mm-ss") & ".avi"
            ScreenRecorder.FileName = Me.txtFileName.Text

            'Start to record
            ScreenRecorder.Record()
        End Using


    End Sub

    Private Sub cmdStopRec_Click(sender As Object, e As EventArgs) Handles cmdStopRec.Click
        'Stop record
        ScreenRecorder.Stop()
    End Sub

    Private Sub cmdPauseIn_Click(sender As Object, e As EventArgs) Handles cmdPauseIn.Click
        'Pause record
        ScreenRecorder.Pause()
    End Sub

    Private Sub cmdUnPauseIn_Click(sender As Object, e As EventArgs) Handles cmdUnPauseIn.Click
        'UnPause record
        ScreenRecorder.UnPause()
    End Sub

    ''' <summary>
    ''' Order a license.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub cmdOrderLicense_Click(sender As Object, e As EventArgs) Handles cmdOrderLicense.Click
        'In order to get your own UserName and RegKey and to distribute this library
        'with your own projects for commercial or any other purpose, please order a license at:
        'https://www.microncode.com/developers/cs-screen-recorder/?cmd=order

        Process.Start("https://www.microncode.com/developers/cs-screen-recorder/?cmd=order")
    End Sub

    Private Class CSharpImpl
        <Obsolete("Please refactor calling code to use normal Visual Basic assignment")>
        Shared Function __Assign(Of T)(ByRef target As T, value As T) As T
            target = value
            Return value
        End Function
    End Class

    Private Sub cboRecorderMode_SelectedIndexChanged(sender As Object, e As EventArgs) Handles cboRecorderMode.SelectedIndexChanged
        Using ScreenRecorder = New ScreenRecorder("UserName", "RegKey")
            'Get the selected capture mode:
            Dim mode As CSScreenAudioRecorder.Mode = [Enum].Parse(GetType(CSScreenAudioRecorder.Mode), Me.cboRecorderMode.Text)

            'Get the audio devices:
            Me.cboAudioSource.DataSource = ScreenRecorder.GetDevices(mode)

            'Try to set the default device:
            Dim default_device_index As Integer = ScreenRecorder.GetDeviceDefaultIndex(mode)

            If default_device_index <> -1 Then
                'Set the default device index:
                Me.cboAudioSource.SelectedIndex = default_device_index
            Else
                'LineIn will always return -1:
                Me.cboAudioSource.SelectedIndex = 0
            End If
        End Using
    End Sub

    Private Sub frmMain_FormClosing(sender As Object, e As FormClosingEventArgs) Handles MyBase.FormClosing
#Region "SafeExit"

        If ScreenRecorder.RecordingState = ScreenRecorder.RecordingScreenState.Recording Then

            ScreenRecorder.Stop()

        End If

#End Region
    End Sub
End Class
